//181027 DesktopStation for Kato SmartController fujigaya2
#include "sc_function.h"
#include "Functions.h"

#define MAX_S88DECODER 1
#define RELPYERROR_300 "300 Command error"
#define RELPYERROR_301 "301 Syntax error"
#define RELPYERROR_302 "302 receive timeout"
#define RELPYERROR_303 "303 Unknown error"
#define RELPYERROR_NONE ""

//Task Schedule
unsigned long gPreviousL1 = 0; //  300ms interval

/* Parser */
String request;
String function;
word arguments[4];
word numOfArguments;
boolean result;

#define LEDSTATE_STOP 0
#define LEDSTATE_RUN 1
#define LEDSTATE_ERR 2

/* LED functions */
byte gLED_State = LEDSTATE_STOP;
byte gLED_Toggle = 0;

void setup() {
  // put your setup code here, to run once:
  //スマートコントローラ側通信
  sc_init_serial();
  
  //PC側通信
  Serial.begin(115200);
  Serial.setTimeout(500);

  Serial.println("--------------------------------------");
  Serial.println("Desktop Station for Smart Controller 181027  ");
  Serial.println("--------------------------------------");
  Serial.println("100 Ready");

  //Reset task
  gPreviousL1 = millis();
}

boolean parse() 
{

  String request = Serial.readStringUntil('\n');

  char aTemp[48];
  request.toCharArray(aTemp, 30);

  int lpar = request.indexOf('(');
  if (lpar == -1) 
  {
    return false;
  }

  function = String(request.substring(0, lpar));
  function.trim();

  int offset = lpar + 1;
  int comma = request.indexOf(',', offset);
  numOfArguments = 0;
  
  while (comma != -1) 
  {
    String tmp = request.substring(offset, comma);
    tmp.trim();
    arguments[numOfArguments++] = stringToWord(tmp);
    offset = comma + 1;
    comma = request.indexOf(',', offset);
  }

  int rpar = request.indexOf(')', offset);
  while (rpar == -1) 
  {
    return false;
  }

  if (rpar > offset) 
  {
    String tmp = request.substring(offset, rpar);
    tmp.trim();
    arguments[numOfArguments++] = stringToWord(tmp);
  }

  return true;
}

boolean dispatch() 
{
  byte aValue;
  byte aValueHigh;
  boolean aResult;
  char aTempStr[16];
  word aLocProtocol;
  byte aDir = 0;
  long aSpeed = 0;
  
  if ((function == "setLocoDirection") || ((function == "sLD")))
  {
    
    if( arguments[1] > 0)
    {
      //FWD=1,REV=2を FWD=0,REV=1に変換
      aDir = arguments[1] - 1;
      
      //異常値はREV固定
      if(aDir > 1)
      {
        aDir = 1;
      }
    }
    else
    {
      //FWD固定
      aDir = 0;
    }
    
    //return DSCore.SetLocoDirection(arguments[0], aDir);
    return sc_SetLocoDirection(arguments[0], aDir);
  
  }
  else if ((function == "setLocoFunction") || ((function == "sLF")))
  {
    //return DSCore.SetLocoFunction(arguments[0], arguments[1] + 1, arguments[2]);
    return sc_SetLocoFunction(arguments[0], arguments[1] + 1, arguments[2]);
  }
  else if ((function == "setTurnout") || ((function == "sTO")))
  {
    //return DSCore.SetTurnout(arguments[0], (byte)arguments[1]);
    return true;
  }
  else if ((function == "setPower") || ((function == "sPW")))
  {
    
    if( arguments[0] == 1)
    {
      /* 状態変更 */
      gLED_State = LEDSTATE_RUN;
    }
    else
    {
      /* 状態変更 */
      gLED_State = LEDSTATE_STOP;
    }
    
    
    /* LED状態の変更 */
    changeLED();
    
    //181027 fujigaya2 電源なし
    //return DSCore.SetPower(arguments[0]);
    return true;
  
  }
  else if ((function == "setLocoSpeed") || ((function == "sLS")))
  {
    //181027 fujigaya2
    //return DSCore.SetLocoSpeedEx(arguments[0], arguments[1], arguments[2]);
    sc_SetLocoSpeedEx(arguments[0], arguments[1], arguments[2]);
  }
  else if ((function == "reset") || ((function == "sRS")))
  {
    Serial.println("100 Ready");
    return true;
  }
  /* reset */
  else if ((function == "setPing") || ((function == "sPI")))
  {
    /* Type(2bytes), Version(2bytes), UID(4bytes) */
    Serial.println("@DSG,00,03,00,02,00,00,00,00");
    return true;
  }
  else if ((function == "getLocoConfig") || ((function == "gCV")))
  {
    
    return true;
  }
  else if (function == "getS88")
  {
    int aMaxS88Num = MAX_S88DECODER;
    
    if( arguments[0] > 0)
    {
      aMaxS88Num = arguments[0];
    }
    
    //Send a S88 sensor reply 
    Serial.print("@S88,");
    //181027 fujigaya2
    //reporter.refresh(aMaxS88Num);

    word aFlags = 0;

    for( int j = 0; j < aMaxS88Num; j++)
    {
      //181027 fujigaya2
      //aFlags = (reporter.getByte((j << 1) + 1) << 8) + reporter.getByte(j << 1);
      //181027 fujigaya2
      //Serial.printHEX((word)aFlags);
      Serial.print("0000");
      Serial.print(",");
    }
      
    Serial.println("");
    
    return true;
  } /* getS88 */
  else if ((function == "setLocoConfig") || ((function == "sCV")))
  {
    
    //181027 fujigaya2
    //return DSCore.WriteConfig_Dir( arguments[1], arguments[2]);
    return true;
  }
  else
  {
    return false;
  }
}


void loop() {
  // put your main code here, to run repeatedly:
  char *aReplyText = RELPYERROR_NONE;
  if( Serial.available() > 0)
  {
    
      if (parse()) {
      if (dispatch()) {
        aReplyText = "200 Ok";
      } else {
        //SerialDS.println(function);
        aReplyText = RELPYERROR_300;
      }
        
    } else {
      aReplyText = RELPYERROR_301;
    }
    
    /* Reply to Desktop Station */
    Serial.println(aReplyText);
    
    Serial.flush();
  }
  else
  {
    if(millis() - gPreviousL1 > 500) {
      sc_idle_packet();
      gPreviousL1 = millis();
    }
  }
  
}


void changeLED(void)
{
  
  
  switch( gLED_State)
  {
  case LEDSTATE_STOP:
    //digitalWrite(PIN_RUNLED, 0);
    
    break;
    
  case LEDSTATE_RUN:
    //digitalWrite(PIN_RUNLED, 1);
    break;
    
  case LEDSTATE_ERR:
    break;
  }
}

void taskLED(void)
{
  
  
  switch( gLED_State)
  {
  case LEDSTATE_STOP:
    break;
    
  case LEDSTATE_RUN:
    break;
    
  case LEDSTATE_ERR:
    //digitalWrite(PIN_RUNLED, gLED_Toggle);
    gLED_Toggle = (~gLED_Toggle) & 1;
    
    break;
  }
}
